@@ -1,11 +1,11 @@
-# $Id: SAX.pm,v 1.26 2006/04/23 23:48:31 matt Exp $
+# $Id: SAX.pm,v 1.30 2008-08-04 21:12:04 grant Exp $
 
 package XML::SAX;
 
 use strict;
 use vars qw($VERSION @ISA @EXPORT_OK);
 
-$VERSION = '0.14';
+$VERSION = '0.95';
 
 use Exporter ();
 @ISA = ('Exporter');
@@ -351,6 +351,10 @@ via ParserFactory:
 
 =head1 AUTHOR
 
+Current maintainer: Grant McLean, grantm@cpan.org
+
+Originally written by:
+
 Matt Sergeant, matt@sergeant.org
 
 Kip Hampton, khampton@totalcinema.com
