@@ -1,17 +1,17 @@
-# $Id: Productions.pm,v 1.9 2002/02/03 12:25:42 matt Exp $
+# $Id: Productions.pm,v 1.12 2008-08-04 10:04:54 grant Exp $
 
 package XML::SAX::PurePerl::Productions;
 
 use Exporter;
 @ISA = ('Exporter');
-@EXPORT_OK = qw($S $Char $VersionNum $BaseChar $Letter $Ideographic
+@EXPORT_OK = qw($S $Char $VersionNum $BaseChar $Ideographic
     $Extender $Digit $CombiningChar $EncNameStart $EncNameEnd $NameChar $CharMinusDash
     $PubidChar $Any $SingleChar);
 
 ### WARNING!!! All productions here must *only* match a *single* character!!! ###
 
 BEGIN {
-$S = qr/^[\x20\x09\x0D\x0A]$/;
+$S = qr/[\x20\x09\x0D\x0A]/;
 
 $CharMinusDash = qr/[^-]/x;
 
@@ -22,7 +22,7 @@ $VersionNum = qr/ [a-zA-Z0-9_.:-]+ /x;
 $EncNameStart = qr/ [A-Za-z] /x;
 $EncNameEnd = qr/ [A-Za-z0-9\._-] /x;
 
-$PubidChar = qr/ [\x20\x0D\x0Aa-zA-Z0-9'()\+,.\/:=\?;!*\#@\$_\%] /x;
+$PubidChar = qr/ [\x20\x0D\x0Aa-zA-Z0-9'()\+,.\/:=\?;!*\#@\$_\%-] /x;
 
 if ($] < 5.006) {
     eval <<'    PERL';
@@ -36,12 +36,10 @@ if ($] < 5.006) {
     
     $Digit = qr/ [\x30-\x39] /x;
     
-    $Letter = qr/^ $BaseChar $/x;
-    
     # can't do this one without unicode
     # $CombiningChar = qr/^$/msx;
     
-    $NameChar = qr/^ $BaseChar | $Digit | [._:-] | $Extender $/x;
+    $NameChar = qr/(?: $BaseChar | $Digit | [._:-] | $Extender )/x;
     PERL
     die $@ if $@;
 }
@@ -138,9 +136,7 @@ else {
 [\x{4E00}-\x{9FA5}\x{3007}\x{3021}-\x{3029}]
 /x;
 
-    $Letter = qr/^ $BaseChar | $Ideographic $/x;
-
-    $NameChar = qr/^ $Letter | $Digit | [._:-] | $CombiningChar | $Extender $/x;
+    $NameChar = qr/(?: $BaseChar | $Ideographic | $Digit | [._:-] | $CombiningChar | $Extender )/x;
     PERL
 
     die $@ if $@;
